/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _949MP7JRSGEOGRAPHICPOINTTYPE_H
#define _949MP7JRSGEOGRAPHICPOINTTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsGeographicPointTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsGeographicPointType_ExtInclude.h


class IMp7JrsGeographicPointType;
class IMp7JrsGeographicPointType_latitude_LocalType;
class IMp7JrsGeographicPointType_longitude_LocalType;
typedef Mp7JrsPtr< IMp7JrsGeographicPointType_latitude_LocalType > Mp7JrsGeographicPointType_latitude_LocalPtr;
typedef Mp7JrsPtr< IMp7JrsGeographicPointType_longitude_LocalType > Mp7JrsGeographicPointType_longitude_LocalPtr;
typedef Mp7JrsPtr< IMp7JrsGeographicPointType> Mp7JrsGeographicPointPtr;

/** 
 * Generated interface IMp7JrsGeographicPointType for class Mp7JrsGeographicPointType<br>
 * Located at: mpeg7-2004.xsd, line 4448<br>
 * Classified: Class<br>
 */
class MP7JRS_EXPORT IMp7JrsGeographicPointType 
 :
        public Mp7JrsNode
{
public:
    // TODO: make these protected?
    IMp7JrsGeographicPointType();
    virtual ~IMp7JrsGeographicPointType();
    /**
     * Get longitude attribute.
     * @return Mp7JrsGeographicPointType_longitude_LocalPtr
     */
    virtual Mp7JrsGeographicPointType_longitude_LocalPtr Getlongitude() const = 0;

    /**
     * Set longitude attribute.
     * @param item const Mp7JrsGeographicPointType_longitude_LocalPtr &
     */
    // Mandatory
    virtual void Setlongitude(const Mp7JrsGeographicPointType_longitude_LocalPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get latitude attribute.
     * @return Mp7JrsGeographicPointType_latitude_LocalPtr
     */
    virtual Mp7JrsGeographicPointType_latitude_LocalPtr Getlatitude() const = 0;

    /**
     * Set latitude attribute.
     * @param item const Mp7JrsGeographicPointType_latitude_LocalPtr &
     */
    // Mandatory
    virtual void Setlatitude(const Mp7JrsGeographicPointType_latitude_LocalPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

    /**
     * Get altitude attribute.
     * @return double
     */
    virtual double Getaltitude() const = 0;

    /**
     * Get altitude attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existaltitude() const = 0;

    /**
     * Set altitude attribute.
     * @param item double
     */
    virtual void Setaltitude(double item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate altitude attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatealtitude() = 0;

    /**
     * Gets or creates Mp7JrsNodePtr child elements of GeographicPointType.
     * Currently just supports rudimentary XPath expressions as GeographicPointType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsGeographicPointType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsGeographicPointType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsGeographicPointType<br>
 * Located at: mpeg7-2004.xsd, line 4448<br>
 * Classified: Class<br>
 * Defined in mpeg7-2004.xsd, line 4448.<br>
 */
class MP7JRS_EXPORT Mp7JrsGeographicPointType :
        public IMp7JrsGeographicPointType
{
    friend class Mp7JrsGeographicPointTypeFactory; // constructs objects

public:
    /*
     * Get longitude attribute.
     * @return Mp7JrsGeographicPointType_longitude_LocalPtr
     */
    virtual Mp7JrsGeographicPointType_longitude_LocalPtr Getlongitude() const;

    /*
     * Set longitude attribute.
     * @param item const Mp7JrsGeographicPointType_longitude_LocalPtr &
     */
    // Mandatory
    virtual void Setlongitude(const Mp7JrsGeographicPointType_longitude_LocalPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get latitude attribute.
     * @return Mp7JrsGeographicPointType_latitude_LocalPtr
     */
    virtual Mp7JrsGeographicPointType_latitude_LocalPtr Getlatitude() const;

    /*
     * Set latitude attribute.
     * @param item const Mp7JrsGeographicPointType_latitude_LocalPtr &
     */
    // Mandatory
    virtual void Setlatitude(const Mp7JrsGeographicPointType_latitude_LocalPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

    /*
     * Get altitude attribute.
     * @return double
     */
    virtual double Getaltitude() const;

    /*
     * Get altitude attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existaltitude() const;

    /*
     * Set altitude attribute.
     * @param item double
     */
    virtual void Setaltitude(double item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate altitude attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatealtitude();

    /*
     * Gets or creates Mp7JrsNodePtr child elements of GeographicPointType.
     * Currently just supports rudimentary XPath expressions as GeographicPointType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsGeographicPointType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsGeographicPointType();

protected:
    Mp7JrsGeographicPointType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
    Mp7JrsGeographicPointType_longitude_LocalPtr m_longitude;
    Mp7JrsGeographicPointType_latitude_LocalPtr m_latitude;
    double m_altitude;
    bool m_altitude_Exist;



// no includefile for extension defined 
// file Mp7JrsGeographicPointType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();
};

#endif // _949MP7JRSGEOGRAPHICPOINTTYPE_H

